
-- Outrun based game

application:setOrientation(Application.LANDSCAPE_LEFT)
--application:setBackgroundColor(0xff0000)

local COLORS = {
  SKY = "0x72D7EE",
  TREE = "0x005108",
  FOG = "0x005108",
  LIGHT = { road = "0x6B6B6B", grass = "0x10AA10", rumble = "0x555555", lane = "0xCCCCCC" },
  DARK = { road = "0x696969", grass = "0x009A00", rumble = "0xBBBBBB" },
  START = { road = "0xffffff", grass = "0xffffff", rumble = "0xffffff" },
  FINISH = { road = "0x000000", grass = "0x000000", rumble = "0x000000" }
}

-- Background, hills and trees
local texture_sky = Texture.new("images/background/sky.png")
local bg_sky = Bitmap.new(texture_sky)

local texture_hills = Texture.new("images/background/hills.png")
local bg_hills = Bitmap.new(texture_hills)

local texture_trees = Texture.new("images/background/trees.png")
local bg_trees = Bitmap.new(texture_trees)

local texture_player = Texture.new("images/player_straight.png")

-- Game variables
local fps = 60 -- how many 'update' frames per second
local step = 1/fps; 
local road_width = 1200
local camera_height = 800
local camera_depth -- this value is calculated
local draw_distance = 100
local rumble_length = 3
local fog_density = 5 --(1-50)

local position = 0 -- Camera position
local speed = 300
local segment_length = 200
local max_speed = segment_length / step
local lanes = 3

local playerX = 0

local width = application:getContentWidth()
local height = application:getContentHeight()

local field_of_view = 100
local camera_depth = 1 / math.tan((field_of_view/2) * math.pi/180);
local playerZ = (camera_height * camera_depth)

local track_length

function increase(start, increment)

	local max = track_length
	local result = start + increment
	
	while (result >= max) do
		result = result - max
	end
	
	while (result < 0) do
		result = result + max
	end
	
	return result
end

function draw_backgrounds()

	stage:addChild(bg_sky)	
	stage:addChild(bg_hills)
	stage:addChild(bg_trees)
end

function reset_road() 
	
	local road_length = 5000
	local segments = {}
		
	for n=1, road_length do
		local segment = {}
		segment.index = n
		
		--Create new segment
		local p1 = {}
		p1.world = { z = (n-1) * segment_length}
		p1.camera = {}
		p1.screen = {}
		segment.p1 = p1
		
		local p2 = {}
		p2.world = { z = (n) * segment_length}
		p2.camera = {}
		p2.screen = {}
		segment.p2 = p2
		
		local color = math.floor(n/rumble_length) % 2
		if (color == 0) then
			segment.color = COLORS.DARK
		else
			segment.color = COLORS.LIGHT
		end
		
		segments[n] = segment
	end
		
	track_length = segment_length * #segments;
	
	return segments
 end

function find_segment(z) 
	
	local index = math.floor(z/segment_length) % #segments
	
	return segments[index + 1]
end

function project(p, cameraX, cameraY, cameraZ, camera_depth, width, height, road_width)
		
	p.camera.x = (p.world.x or 0) - cameraX
	p.camera.y = (p.world.y or 0) - cameraY
	p.camera.z = (p.world.z or 0) - cameraZ
	
	local scale = camera_depth / p.camera.z
	p.screen.scale = scale
	p.screen.x = math.ceil((width / 2) + (scale * p.camera.x * width / 2))
	p.screen.y = math.ceil((height / 2) - (scale * p.camera.y * height / 2))
	p.screen.w = math.ceil(p.screen.scale * road_width * width/2)
	
end

Render = {}

Render.segments = {}
 
function Render.rectangle(x, y, width, height, color)

	local shape = Shape.new()
	
	shape:setFillStyle(Shape.SOLID, color.grass)
	shape:beginPath(Shape.NON_ZERO)
	shape:moveTo(x, y)
	shape:lineTo(x + width, y)
	shape:lineTo(x + width, y + height)
	shape:lineTo(x, y + height)
	shape:lineTo(x, y)
	shape:endPath()
	--stage:addChild(shape)
	return shape
	
end

function Render.polygon(x1, y1, x2, y2, x3, y3, x4, y4, color)

	local shape = Shape.new()
	
	shape:setFillStyle(Shape.SOLID, color) 
	--shape:setLineStyle(1, 0xff0000)
	shape:beginPath(Shape.NON_ZERO)
	shape:moveTo(x1, y1)
	shape:lineTo(x2, y2)
	shape:lineTo(x3, y3)
	shape:lineTo(x4, y4)
	shape:endPath()
	
	--stage:addChild(shape)
	return shape
  
end

function Render.segment (x1, y1, w1, x2, y2, w2, fog, color) 
		
	local r1 = Render.rumbleWidth(w1)
	local r2 = Render.rumbleWidth(w2)
	local l1 = Render.laneMarkerWidth(w1)
	local l2 = Render.laneMarkerWidth(w2)
	
	local group = Sprite.new()
	--local rect = Render.rectangle(0, y2, width, y1 - y2, color)
	local rect = Rectangle.new(0, y2, width, y1 - y2, color)
	group:addChild(rect)
	--local polygon1 = Render.polygon(x1-w1-r1, y1, x1-w1, y1, x2-w2, y2, x2-w2-r2, y2, color.rumble)
	local polygon1 = Polygon.new(x1-w1-r1, y1, x1-w1, y1, x2-w2, y2, x2-w2-r2, y2, color.rumble)
	group:addChild(polygon1)
	local polygon2 = Render.polygon(x1+w1+r1, y1, x1+w1, y1, x2+w2, y2, x2+w2+r2, y2, color.rumble)
	group:addChild(polygon2)
    local polygon3 = Render.polygon(x1-w1, y1, x1+w1, y1, x2+w2, y2, x2-w2, y2, color.road)
	group:addChild(polygon3)
	
	if (color.lane) then
		local lane_w1 = w1*2/lanes
		local lane_w2 = w2*2/lanes
		local lane_x1 = x1 - w1 + lane_w1
		local lane_x2 = x2 - w2 + lane_w2
		
		for lane =1, lanes-1 do
			local polygon_lane = Render.polygon(lane_x1 - l1/2, y1, lane_x1 + l1/2, y1, 
							lane_x2 + l2/2, y2, lane_x2 - l2/2, y2, color.lane)
			lane_x1 = lane_x1 + lane_w1
			lane_x2 = lane_x2 + lane_w2
			group:addChild(polygon_lane)
		end
		
	end
	
	return group

end


function Render.rumbleWidth(projectedRoadWidth)
	return projectedRoadWidth/math.max(6, 2*lanes)
end

function Render.laneMarkerWidth(projectedRoadWidth)
	return projectedRoadWidth/math.max(32, 8*lanes) 
end

--[[
   fog: function(ctx, x, y, width, height, fog) {
    if (fog < 1) {
      ctx.globalAlpha = (1-fog)
      ctx.fillStyle = COLORS.FOG;
      ctx.fillRect(x, y, width, height);
      ctx.globalAlpha = 1;
    }
]]--

function draw_road()

	local segments = segments
	local base_segment = find_segment(position)
	local maxy = height
	local num_segments = #segments
	
	--local t1= os.clock()
	
	local j = 1
	for i = 0, draw_distance -1 do
		local index = (base_segment.index + i) % num_segments
					
		local segment = segments[index + 1]
		local p1 = segment.p1
		local p2 = segment.p2
		
		project(p1, playerX * road_width, camera_height, position, camera_depth, 
				width, height, road_width)
				
		project(p2, playerX * road_width, camera_height, position, camera_depth, 
				width, height, road_width)
			
		if not (segment.p1.camera.z <= camera_depth or -- behind us
			segment.p2.screen.y >= maxy) then       -- clip by (already rendered) segment
	
			--local t1= os.clock()
			--print (j)
			
			local sprite_segment = Render.segment(
							p1.screen.x,
							p1.screen.y,
							p1.screen.w, 
							p2.screen.x,
							p2.screen.y,
							p2.screen.w,
							segment.fog,
							segment.color)
			stage:addChild(sprite_segment)
			Render.segments[j] = sprite_segment
			j = j + 1
		
			maxy = segment.p2.screen.y
			
			--local t2 = os.clock() - t1
			--print (t2)
		end
	end
	
	--local t2 = os.clock() - t1
	--print (t2)
end

-- init
draw_backgrounds()
segments = reset_road()
draw_road()

-- Player
local player = Bitmap.new(texture_player)
player:setScale(2.5)
local posX = (width - player:getWidth()) * 0.5
local posY = height - player:getHeight()
player:setPosition(posX, posY)
stage:addChild(player)

-- Game loop
function update()
		
	--[[
	local sprite_segments = Render.segments
	
	for i=1,sprite_segments do
		local sprite = sprite_segments[i]
	end
	]]--
	
	position = increase(position, speed)
	draw_road()
	
	--player position
	local player = Bitmap.new(texture_player)
	player:setScale(2.5)
	local posX = (width - player:getWidth()) * 0.5
	local posY = height - player:getHeight()
	player:setPosition(posX, posY)
	stage:addChild(player)

end

-- Add event listener for loop game
stage:addEventListener(Event.ENTER_FRAME, update)

